/**
 * @file main.cpp
 * @brief 
 */

#include <iostream>
#include <iomanip>
#include "cpxmacro.h"

#include "cs1d.h"

using namespace std;

// error status and messagge buffer
int status;
char errmsg[BUF_SIZE];

int main (int argc, char const *argv[])
{
	try
	{
		DECL_ENV( env );
		DECL_ENV( env2 );
		DECL_PROB( env, lp );
		CS1D cs1dSolver(env, lp);
		Data data;
		data.read(argv[1]);
		cs1dSolver.initMaster(data);
		std::cout << std::endl << "Solving LP" << std::endl << "*** It ***\tObj\tx" << std::endl;
		int it = 1;
		std::vector<double> x;
		std::vector<double> u;
		double objval;
		while(true)
		{
			cs1dSolver.solveMasterLP(x, u, objval);
			std::cout << "*** " << it++ << " ***\t" << objval << "\t";
			for (unsigned int i = 0; i < x.size(); i++) std::cout << setw(5) << x[i] << " ";
			std::cout << std::endl;
			bool newCol = cs1dSolver.price(env2, data, u);
			if (!newCol) break;
		}
		// final B&C
		std::cout << "LP value: " << objval << std::endl;
		std::cout << "\nB&C... " << std::endl;
		cs1dSolver.branchAndCut(x, objval);
		std::cout << "x = ";
		for (unsigned int i = 0; i < x.size(); i++) std::cout << setw(5) << x[i] << " ";
		std::cout << std::endl;
		std::cout << "B&C value: " << objval << std::endl;
		
		//debug
		CHECKED_CPX_CALL( CPXwriteprob, env, lp, "final.lp", NULL );
		
		// free
		CPXfreeprob(env, &lp);
		CPXcloseCPLEX(&env);
		CPXcloseCPLEX(&env2);
	}
	catch(std::exception& e)
	{
		std::cout << ">>>EXCEPTION: " << e.what() << std::endl;
	}
	return 0;
}
